@@ -10,14 +10,14 @@ from airflow.configuration import conf
 from airflow.executors.base_executor import BaseExecutor
 from airflow.utils.module_loading import import_string
 from airflow.utils.state import State
-from marshmallow import Schema, fields, post_load
+from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load
 
 CommandType = List[str]
 TaskInstanceKeyType = Tuple[Any]
 ExecutorConfigFunctionType = Callable[[CommandType], dict]
-EcsFargateQueuedTask = namedtuple('EcsFargateQueuedTask', ('key', 'command', 'executor_config'))
+EcsFargateQueuedTask = namedtuple('EcsFargateQueuedTask', ('key', 'command', 'queue', 'executor_config'))
 ExecutorConfigType = Dict[str, Any]
-EcsFargateTaskInfo = namedtuple('EcsFargateTaskInfo', ('cmd', 'config'))
+EcsFargateTaskInfo = namedtuple('EcsFargateTaskInfo', ('cmd', 'queue', 'config'))
 
 
 class EcsFargateTask:
@@ -147,17 +147,18 @@ class AwsEcsFargateExecutor(BaseExecutor):
         for i in range((len(task_arns) // self.DESCRIBE_TASKS_BATCH_SIZE) + 1):
             batched_task_arns = task_arns[i * self.DESCRIBE_TASKS_BATCH_SIZE: (i + 1) * self.DESCRIBE_TASKS_BATCH_SIZE]
             boto_describe_tasks = self.ecs.describe_tasks(tasks=batched_task_arns, cluster=self.cluster)
-            describe_tasks_response = BotoDescribeTasksSchema().load(boto_describe_tasks)
-            if describe_tasks_response.errors:
+            try:
+                describe_tasks_response = BotoDescribeTasksSchema().load(boto_describe_tasks)
+            except ValidationError as err:
                 self.log.error('ECS DescribeTask Response: %s', boto_describe_tasks)
                 raise EcsFargateError(
                     'DescribeTasks API call does not match expected JSON shape. '
                     'Are you sure that the correct version of Boto3 is installed? {}'.format(
-                        describe_tasks_response.errors
+                        err
                     )
                 )
-            all_task_descriptions['tasks'].extend(describe_tasks_response.data['tasks'])
-            all_task_descriptions['failures'].extend(describe_tasks_response.data['failures'])
+            all_task_descriptions['tasks'].extend(describe_tasks_response['tasks'])
+            all_task_descriptions['failures'].extend(describe_tasks_response['failures'])
         return all_task_descriptions
 
     def __handle_failed_task(self, task_arn: str, reason: str):
@@ -166,14 +167,14 @@ class AwsEcsFargateExecutor(BaseExecutor):
         ECS/Fargate Cloud. If an API failure occurs the task is simply rescheduled.
         """
         task_key = self.active_workers.arn_to_key[task_arn]
-        task_cmd, exec_info = self.active_workers.info_by_key(task_key)
+        task_cmd, queue, exec_info = self.active_workers.info_by_key(task_key)
         failure_count = self.active_workers.failure_count_by_key(task_key)
         if failure_count < self.__class__.MAX_FAILURE_CHECKS:
             self.log.warning('Task %s has failed due to %s. '
                              'Failure %s out of %s occurred on %s. Rescheduling.',
                              task_key, reason, failure_count, self.__class__.MAX_FAILURE_CHECKS, task_arn)
             self.active_workers.increment_failure_count(task_key)
-            self.pending_tasks.appendleft(EcsFargateQueuedTask(task_key, task_cmd, exec_info))
+            self.pending_tasks.appendleft(EcsFargateQueuedTask(task_key, task_cmd, queue, exec_info))
         else:
             self.log.error('Task %s has failed a maximum of %s times. Marking as failed', task_key,
                            failure_count)
@@ -192,8 +193,8 @@ class AwsEcsFargateExecutor(BaseExecutor):
         failure_reasons = defaultdict(int)
         for _ in range(queue_len):
             ecs_task = self.pending_tasks.popleft()
-            task_key, cmd, exec_config = ecs_task
-            run_task_response = self.__run_task(cmd, exec_config)
+            task_key, cmd, queue, exec_config = ecs_task
+            run_task_response = self._run_task(task_key, cmd, queue, exec_config)
             if run_task_response['failures']:
                 for f in run_task_response['failures']:
                     failure_reasons[f['reason']] += 1
@@ -203,39 +204,53 @@ class AwsEcsFargateExecutor(BaseExecutor):
                 raise EcsFargateError('No failures and no tasks provided in response. This should never happen.')
             else:
                 task = run_task_response['tasks'][0]
-                self.active_workers.add_task(task, task_key, cmd, exec_config)
+                self.active_workers.add_task(task, task_key, queue, cmd, exec_config)
         if failure_reasons:
             self.log.debug('Pending tasks failed to launch for the following reasons: %s. Will retry later.',
                            dict(failure_reasons))
 
-    def __run_task(self, cmd: CommandType, exec_config: ExecutorConfigType):
+    def _run_task(self, task_id: TaskInstanceKeyType, cmd: CommandType, queue: str, exec_config: ExecutorConfigType):
         """
+        This function is the actual attempt to run a queued-up airflow task. Not to be confused with
+        execute_async() which inserts tasks into the queue.
         The command and executor config will be placed in the container-override section of the JSON request, before
         calling Boto3's "run_task" function.
         """
+        run_task_api = self._run_task_kwargs(task_id, cmd, queue, exec_config)
+        boto_run_task = self.ecs.run_task(**run_task_api)
+        try:
+            run_task_response = BotoRunTaskSchema().load(boto_run_task)
+        except ValidationError as err:
+            self.log.error('ECS RunTask Response: %s', err)
+            raise EcsFargateError(
+                'RunTask API call does not match expected JSON shape. '
+                'Are you sure that the correct version of Boto3 is installed? {}'.format(
+                    err
+                )
+            )
+        return run_task_response
+
+    def _run_task_kwargs(self, task_id: TaskInstanceKeyType, cmd: CommandType,
+                         queue: str, exec_config: ExecutorConfigType) -> dict:
+        """
+        This modifies the standard kwargs to be specific to this task by overriding the airflow command and updating
+        the container overrides.
+
+        One last chance to modify Boto3's "run_task" kwarg params before it gets passed into the Boto3 client.
+        """
         run_task_api = deepcopy(self.run_task_kwargs)
         container_override = self.get_container(run_task_api['overrides']['containerOverrides'])
         container_override['command'] = cmd
         container_override.update(exec_config)
-        boto_run_task = self.ecs.run_task(**run_task_api)
-        run_task_response = BotoRunTaskSchema().load(boto_run_task)
-        if run_task_response.errors:
-            self.log.error('ECS RunTask Response: %s', run_task_response)
-            raise EcsFargateError(
-                'RunTask API call does not match expected JSON shape. '
-                'Are you sure that the correct version of Boto3 is installed? {}'.format(
-                    run_task_response.errors
-                )
-            )
-        return run_task_response.data
+        return run_task_api
 
     def execute_async(self, key: TaskInstanceKeyType, command: CommandType, queue=None, executor_config=None):
         """
-        Save the task to be executed in the next sync using Boto3's RunTask API
+        Save the task to be executed in the next sync by inserting the commands into a queue.
         """
         if executor_config and ('name' in executor_config or 'command' in executor_config):
             raise ValueError('Executor Config should never override "name" or "command"')
-        self.pending_tasks.append(EcsFargateQueuedTask(key, command, executor_config or {}))
+        self.pending_tasks.append(EcsFargateQueuedTask(key, command, queue, executor_config or {}))
 
     def end(self, heartbeat_interval=10):
         """
@@ -298,14 +313,14 @@ class EcsFargateTaskCollection:
         self.key_to_failure_counts: Dict[TaskInstanceKeyType, int] = defaultdict(int)
         self.key_to_task_info: Dict[TaskInstanceKeyType, EcsFargateTaskInfo] = {}
 
-    def add_task(self, task: EcsFargateTask, airflow_task_key: TaskInstanceKeyType, airflow_cmd: CommandType,
-                 exec_config: ExecutorConfigType):
+    def add_task(self, task: EcsFargateTask, airflow_task_key: TaskInstanceKeyType, queue: str,
+                 airflow_cmd: CommandType, exec_config: ExecutorConfigType):
         """Adds a task to the collection"""
         arn = task.task_arn
         self.tasks[arn] = task
         self.key_to_arn[airflow_task_key] = arn
         self.arn_to_key[arn] = airflow_task_key
-        self.key_to_task_info[airflow_task_key] = EcsFargateTaskInfo(airflow_cmd, exec_config)
+        self.key_to_task_info[airflow_task_key] = EcsFargateTaskInfo(airflow_cmd, queue, exec_config)
 
     def update_task(self, task: EcsFargateTask):
         """Updates the state of the given task based on task ARN"""
@@ -366,29 +381,34 @@ class BotoContainerSchema(Schema):
     Botocore Serialization Object for ECS 'Container' shape.
     Note that there are many more parameters, but the executor only needs the members listed below.
     """
-    exit_code = fields.Integer(load_from='exitCode')
-    last_status = fields.String(load_from='lastStatus')
+    exit_code = fields.Integer(data_key='exitCode')
+    last_status = fields.String(data_key='lastStatus')
     name = fields.String(required=True)
 
+    class Meta:
+        unknown = EXCLUDE
+
 
 class BotoTaskSchema(Schema):
     """
     Botocore Serialization Object for ECS 'Task' shape.
     Note that there are many more parameters, but the executor only needs the members listed below.
     """
-    task_arn = fields.String(load_from='taskArn', required=True)
-    last_status = fields.String(load_from='lastStatus', required=True)
-    desired_status = fields.String(load_from='desiredStatus', required=True)
+    task_arn = fields.String(data_key='taskArn', required=True)
+    last_status = fields.String(data_key='lastStatus', required=True)
+    desired_status = fields.String(data_key='desiredStatus', required=True)
     containers = fields.List(fields.Nested(BotoContainerSchema), required=True)
-    started_at = fields.Field(load_from='startedAt')
-    stopped_reason = fields.String(load_from='stoppedReason')
+    started_at = fields.Field(data_key='startedAt')
+    stopped_reason = fields.String(data_key='stoppedReason')
 
     @post_load
     def make_task(self, data, **kwargs):
-        """Overwrites marshmallow .data property to return an instance of EcsFargateTask instead of a dictionary"""
-        print('postload task schema')
+        """Overwrites marshmallow load() to return an instance of EcsFargateTask instead of a dictionary"""
         return EcsFargateTask(**data)
 
+    class Meta:
+        unknown = EXCLUDE
+
 
 class BotoFailureSchema(Schema):
     """
@@ -397,6 +417,9 @@ class BotoFailureSchema(Schema):
     arn = fields.String()
     reason = fields.String()
 
+    class Meta:
+        unknown = EXCLUDE
+
 
 class BotoRunTaskSchema(Schema):
     """
@@ -405,6 +428,9 @@ class BotoRunTaskSchema(Schema):
     tasks = fields.List(fields.Nested(BotoTaskSchema), required=True)
     failures = fields.List(fields.Nested(BotoFailureSchema), required=True)
 
+    class Meta:
+        unknown = EXCLUDE
+
 
 class BotoDescribeTasksSchema(Schema):
     """
@@ -413,6 +439,9 @@ class BotoDescribeTasksSchema(Schema):
     tasks = fields.List(fields.Nested(BotoTaskSchema), required=True)
     failures = fields.List(fields.Nested(BotoFailureSchema), required=True)
 
+    class Meta:
+        unknown = EXCLUDE
+
 
 class EcsFargateError(Exception):
     """Thrown when something unexpected has occurred within the AWS ECS/Fargate ecosystem"""
